@@ -231,7 +231,7 @@ static bool s_parse_retryable_error_from_response(struct aws_allocator *allocato
     if (xml_parser == NULL) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "Failed to init xml parser for sts web identity credentials provider to parse error information.")
+            "Failed to init xml parser for sts web identity credentials provider to parse error information.");
         return false;
     }
     bool get_retryable_error = false;
@@ -330,14 +330,14 @@ static struct aws_credentials *s_parse_credentials_from_response(
     if (xml_parser == NULL) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "Failed to init xml parser for sts web identity credentials provider to parse error information.")
+            "Failed to init xml parser for sts web identity credentials provider to parse error information.");
         return NULL;
     }
     uint64_t now = UINT64_MAX;
     if (aws_sys_clock_get_ticks(&now) != AWS_OP_SUCCESS) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "Failed to get sys clock for sts web identity credentials provider to parse error information.")
+            "Failed to get sys clock for sts web identity credentials provider to parse error information.");
         goto on_finish;
     }
     uint64_t now_seconds = aws_timestamp_convert(now, AWS_TIMESTAMP_NANOS, AWS_TIMESTAMP_SECS, NULL);
@@ -846,7 +846,6 @@ on_error:
     return NULL;
 }
 
-static struct aws_byte_cursor s_default_profile_name_cursor = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("default");
 static struct aws_byte_cursor s_dot_cursor = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(".");
 static struct aws_byte_cursor s_amazonaws_cursor = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(".amazonaws.com");
 static struct aws_byte_cursor s_cn_cursor = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(".cn");
@@ -957,7 +956,9 @@ static void s_parameters_destroy(struct sts_web_identity_parameters *parameters)
     aws_mem_release(parameters->allocator, parameters);
 }
 
-static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator *allocator) {
+static struct sts_web_identity_parameters *s_parameters_new(
+    struct aws_allocator *allocator,
+    struct aws_profile_collection *config_profile_collection_cached) {
 
     struct sts_web_identity_parameters *parameters =
         aws_mem_calloc(allocator, 1, sizeof(struct sts_web_identity_parameters));
@@ -988,9 +989,19 @@ static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator
     bool get_all_parameters =
         (region && region->len && role_arn && role_arn->len && token_file_path && token_file_path->len);
     if (!get_all_parameters) {
-        config_profile = s_load_profile(allocator);
-        profile_name = aws_get_profile_name(allocator, &s_default_profile_name_cursor);
-        if (config_profile && profile_name) {
+        if (config_profile_collection_cached) {
+            /* Use cached profile collection */
+            config_profile = aws_profile_collection_acquire(config_profile_collection_cached);
+        } else {
+            /* Load profile collection from files */
+            config_profile = s_load_profile(allocator);
+            if (!config_profile) {
+                goto on_finish;
+            }
+        }
+
+        profile_name = aws_get_profile_name(allocator, NULL);
+        if (profile_name) {
             profile = aws_profile_collection_get_profile(config_profile, profile_name);
         }
 
@@ -998,7 +1009,7 @@ static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator
             AWS_LOGF_ERROR(
                 AWS_LS_AUTH_CREDENTIALS_PROVIDER,
                 "Failed to resolve either region, role arn or token file path during sts web identity provider "
-                "initialization.")
+                "initialization.");
             goto on_finish;
 
         } else {
@@ -1021,7 +1032,7 @@ static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator
         aws_byte_buf_init_copy_from_cursor(&parameters->role_arn, allocator, aws_byte_cursor_from_string(role_arn))) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "Failed to resolve role arn during sts web identity provider initialization.")
+            "Failed to resolve role arn during sts web identity provider initialization.");
         goto on_finish;
     }
 
@@ -1031,7 +1042,7 @@ static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator
             &parameters->token_file_path, allocator, aws_byte_cursor_from_string(token_file_path))) {
         AWS_LOGF_ERROR(
             AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-            "Failed to resolve token file path during sts web identity provider initialization.")
+            "Failed to resolve token file path during sts web identity provider initialization.");
         goto on_finish;
     }
 
@@ -1047,7 +1058,7 @@ static struct sts_web_identity_parameters *s_parameters_new(struct aws_allocator
 
     AWS_LOGF_DEBUG(
         AWS_LS_AUTH_CREDENTIALS_PROVIDER,
-        "Successfully loaded all required parameters for sts web identity credentials provider.")
+        "Successfully loaded all required parameters for sts web identity credentials provider.");
     success = true;
 
 on_finish:
@@ -1056,7 +1067,7 @@ on_finish:
     aws_string_destroy(role_session_name);
     aws_string_destroy(token_file_path);
     aws_string_destroy(profile_name);
-    aws_profile_collection_destroy(config_profile);
+    aws_profile_collection_release(config_profile);
     if (!success) {
         s_parameters_destroy(parameters);
         parameters = NULL;
@@ -1068,7 +1079,8 @@ struct aws_credentials_provider *aws_credentials_provider_new_sts_web_identity(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_sts_web_identity_options *options) {
 
-    struct sts_web_identity_parameters *parameters = s_parameters_new(allocator);
+    struct sts_web_identity_parameters *parameters =
+        s_parameters_new(allocator, options->config_profile_collection_cached);
     if (!parameters) {
         return NULL;
     }
