@@ -5,11 +5,9 @@
 
 #include <aws/mqtt/mqtt.h>
 
-#include <aws/io/logging.h>
+#include <aws/http/http.h>
 
-#ifdef AWS_MQTT_WITH_WEBSOCKETS
-#    include <aws/http/http.h>
-#endif
+#include <aws/io/logging.h>
 
 /*******************************************************************************
  * Topic Validation
@@ -216,6 +214,12 @@ bool aws_mqtt_is_valid_topic_filter(const struct aws_byte_cursor *topic_filter)
             AWS_DEFINE_ERROR_INFO_MQTT(
                 AWS_ERROR_MQTT5_INVALID_OUTBOUND_TOPIC_ALIAS,
                 "Outgoing publish contained an invalid (too large or unknown) topic alias"),
+            AWS_DEFINE_ERROR_INFO_MQTT(
+                AWS_ERROR_MQTT5_INVALID_UTF8_STRING,
+                "Outbound packet contains invalid utf-8 data in a field that must be utf-8"),
+            AWS_DEFINE_ERROR_INFO_MQTT(
+                AWS_ERROR_MQTT_CONNECTION_RESET_FOR_ADAPTER_CONNECT,
+                "Mqtt5 connection was reset by the Mqtt3 adapter in order to guarantee correct connection configuration"),
         };
 /* clang-format on */
 #undef AWS_DEFINE_ERROR_INFO_MQTT
@@ -250,9 +254,8 @@ void aws_mqtt_library_init(struct aws_allocator *allocator) {
     if (!s_mqtt_library_initialized) {
         s_mqtt_library_initialized = true;
         aws_io_library_init(allocator);
-#ifdef AWS_MQTT_WITH_WEBSOCKETS
         aws_http_library_init(allocator);
-#endif
+
         aws_register_error_info(&s_error_list);
         aws_register_log_subject_info_list(&s_logging_subjects_list);
     }
@@ -264,9 +267,8 @@ void aws_mqtt_library_clean_up(void) {
         aws_thread_join_all_managed();
         aws_unregister_error_info(&s_error_list);
         aws_unregister_log_subject_info_list(&s_logging_subjects_list);
-#ifdef AWS_MQTT_WITH_WEBSOCKETS
+
         aws_http_library_clean_up();
-#endif
         aws_io_library_clean_up();
     }
 }
