@@ -22,7 +22,6 @@ struct aws_s3_client;
 struct aws_s3_connection;
 struct aws_s3_meta_request;
 struct aws_s3_request;
-struct aws_s3_request_options;
 struct aws_http_headers;
 struct aws_http_make_request_options;
 struct aws_retry_strategy;
@@ -45,10 +44,14 @@ typedef void(aws_s3_meta_request_prepare_request_callback_fn)(
     void *user_data);
 
 struct aws_s3_prepare_request_payload {
+    struct aws_allocator *allocator;
     struct aws_s3_request *request;
+    struct aws_task task;
+    /* async step: wait for vtable->prepare_request() call to complete */
+    struct aws_future_void *asyncstep_prepare_request;
+    /* callback to invoke when all request preparation work is complete */
     aws_s3_meta_request_prepare_request_callback_fn *callback;
     void *user_data;
-    struct aws_task task;
 };
 
 struct aws_s3_meta_request_vtable {
@@ -56,15 +59,19 @@ struct aws_s3_meta_request_vtable {
      * progress, false if there is not. */
     bool (*update)(struct aws_s3_meta_request *meta_request, uint32_t flags, struct aws_s3_request **out_request);
 
+    /* Run vtable->prepare_request() on the meta-request's event loop.
+     * We do this because body streaming is slow, and we don't want it on our networking threads.
+     * The callback may fire on any thread (an async sub-step may run on another thread). */
     void (*schedule_prepare_request)(
         struct aws_s3_meta_request *meta_request,
         struct aws_s3_request *request,
         aws_s3_meta_request_prepare_request_callback_fn *callback,
         void *user_data);
 
-    /* Given a request, prepare it for sending (ie: creating the correct HTTP message, reading from a stream (if
-     * necessary), signing it, computing hashes, etc.) */
-    int (*prepare_request)(struct aws_s3_meta_request *meta_request, struct aws_s3_request *request);
+    /* Given a request, asynchronously prepare it for sending
+     * (creating the correct HTTP message, reading from a stream (if necessary), computing hashes, etc.).
+     * Returns a future, which may complete on any thread (and may complete synchronously). */
+    struct aws_future_void *(*prepare_request)(struct aws_s3_request *request);
 
     void (*init_signing_date_time)(struct aws_s3_meta_request *meta_request, struct aws_date_time *date_time);
 
@@ -88,7 +95,10 @@ struct aws_s3_meta_request_vtable {
     void (*destroy)(struct aws_s3_meta_request *);
 
     /* Pause the given request */
-    int (*pause)(struct aws_s3_meta_request *meta_request, struct aws_string **resume_token);
+    int (*pause)(struct aws_s3_meta_request *meta_request, struct aws_s3_meta_request_resume_token **resume_token);
+
+    /* Get the type of the aws_s3_request */
+    int (*get_request_type)(struct aws_s3_request *request);
 };
 
 /**
@@ -107,6 +117,10 @@ struct aws_s3_meta_request {
     /* Initial HTTP Message that this meta request is based on. */
     struct aws_http_message *initial_request_message;
 
+    /* Async stream for meta request's body.
+     * NULL if using initial_request_message's synchronous body stream instead.  */
+    struct aws_async_input_stream *request_body_async_stream;
+
     /* Part size to use for uploads and downloads.  Passed down by the creating client. */
     const size_t part_size;
 
@@ -118,7 +132,7 @@ struct aws_s3_meta_request {
 
     struct aws_s3_endpoint *endpoint;
 
-    /* Event loop to schedule IO work related on, ie, reading from streams, streaming parts back to the caller, etc..
+    /* Event loop to schedule IO work related on, ie, reading from streams, streaming parts back to the caller, etc...
      * After the meta request is finished, this will be reset along with the client reference.*/
     struct aws_event_loop *io_event_loop;
 
@@ -132,6 +146,8 @@ struct aws_s3_meta_request {
     aws_s3_meta_request_shutdown_fn *shutdown_callback;
     aws_s3_meta_request_progress_fn *progress_callback;
 
+    aws_s3_meta_request_telemetry_fn *telemetry_callback;
+
     /* Customer specified callbacks to be called by our specialized callback to calculate the response checksum. */
     aws_s3_meta_request_headers_callback_fn *headers_user_callback_after_checksum;
     aws_s3_meta_request_receive_body_callback_fn *body_user_callback_after_checksum;
@@ -152,8 +168,8 @@ struct aws_s3_meta_request {
         /* The sum of initial_read_window, plus all window_increment() calls. This number never goes down. */
         uint64_t read_window_running_total;
 
-        /* The next expected streaming part number needed to continue streaming part bodies.  (For example, this will
-         * initially be 1 for part 1, and after that part is received, it will be 2, then 3, etc.. */
+        /* The next expected streaming part number needed to continue streaming part bodies. (For example, this will
+         * initially be 1 for part 1, and after that part is received, it will be 2, then 3, etc.. )*/
         uint32_t next_streaming_part;
 
         /* Number of parts scheduled for delivery. */
@@ -194,10 +210,10 @@ struct aws_s3_meta_request {
     /* deep copy of the checksum config. */
     struct checksum_config checksum_config;
 
-    /* checksum found in either a default get request, or in the initial head request of a mutlipart get */
+    /* checksum found in either a default get request, or in the initial head request of a multipart get */
     struct aws_byte_buf meta_request_level_response_header_checksum;
 
-    /* running checksum of all of the parts of a default get, or ranged get meta request*/
+    /* running checksum of all the parts of a default get, or ranged get meta request*/
     struct aws_s3_checksum *meta_request_level_running_response_sum;
 };
 
@@ -264,14 +280,20 @@ void aws_s3_meta_request_sign_request_default(
     aws_signing_complete_fn *on_signing_complete,
     void *user_data);
 
-/* Default implementation for when a request finishes a particular send. Will be invoked for each send of the request.
- */
+/* Default implementation for when a request finishes a particular send. */
 AWS_S3_API
 void aws_s3_meta_request_send_request_finish_default(
     struct aws_s3_connection *connection,
     struct aws_http_stream *stream,
     int error_code);
 
+/* Implementation for when a request finishes a particular send to handle possible async error from S3. */
+AWS_S3_API
+void aws_s3_meta_request_send_request_finish_handle_async_error(
+    struct aws_s3_connection *connection,
+    struct aws_http_stream *stream,
+    int error_code);
+
 /* Called by the client when a request is completely finished and not doing any further retries. */
 AWS_S3_API
 void aws_s3_meta_request_finished_request(
@@ -281,16 +303,25 @@ void aws_s3_meta_request_finished_request(
 
 /* Called to place the request in the meta request's priority queue for streaming back to the caller.  Once all requests
  * with a part number less than the given request has been received, the given request and the previous requests will
- * scheduled for streaming.  */
+ * be scheduled for streaming.  */
 AWS_S3_API
 void aws_s3_meta_request_stream_response_body_synced(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *request);
 
-/* Read from the meta request's input stream. Should always be done outside of any mutex, as reading from the stream
- * could cause user code to call back into aws-c-s3.*/
+/* Asynchronously read from the meta request's input stream. Should always be done outside of any mutex,
+ * as reading from the stream could cause user code to call back into aws-c-s3.
+ * This will fill the buffer to capacity, unless end of stream is reached.
+ * It may read from the underlying stream multiple times, if that's what it takes to fill the buffer.
+ * Returns a future whose result bool indicates whether end of stream was reached.
+ * This future may complete on any thread, and may complete synchronously.
+ */
 AWS_S3_API
-int aws_s3_meta_request_read_body(struct aws_s3_meta_request *meta_request, struct aws_byte_buf *buffer);
+struct aws_future_bool *aws_s3_meta_request_read_body(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_byte_buf *buffer);
+
+bool aws_s3_meta_request_body_has_no_more_data(const struct aws_s3_meta_request *meta_request);
 
 /* Set the meta request finish result as failed. This is meant to be called sometime before aws_s3_meta_request_finish.
  * Subsequent calls to this function or to aws_s3_meta_request_set_success_synced will not overwrite the end result of
@@ -317,7 +348,7 @@ bool aws_s3_meta_request_has_finish_result_synced(struct aws_s3_meta_request *me
 AWS_S3_API
 void aws_s3_meta_request_finish(struct aws_s3_meta_request *meta_request);
 
-/* Default implementation of the meta request finish functino. */
+/* Default implementation of the meta request finish function. */
 AWS_S3_API
 void aws_s3_meta_request_finish_default(struct aws_s3_meta_request *meta_request);
 
