@@ -18,6 +18,103 @@
 #define DEFINE_HEADER(NAME, VALUE)                                                                                     \
     { .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(NAME), .value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(VALUE), }
 
+static int s_validate_mpu_mock_server_metrics(struct aws_array_list *metrics_list) {
+    /* Check the size of the metrics should be the same as the number of requests, which should be create MPU, two
+     * upload parts and one complete MPU */
+    ASSERT_UINT_EQUALS(aws_array_list_length(metrics_list), 4);
+    struct aws_s3_request_metrics *metrics = NULL;
+
+    /* First metrics should be the CreateMPU */
+    aws_array_list_get_at(metrics_list, (void **)&metrics, 0);
+    struct aws_http_headers *response_headers = NULL;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_response_headers(metrics, &response_headers));
+    const struct aws_string *request_id = NULL;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_request_id(metrics, &request_id));
+    ASSERT_TRUE(aws_string_eq_c_str(request_id, "12345"));
+    const struct aws_string *ip_address = NULL;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_ip_address(metrics, &ip_address));
+    /* Should be default local ip for ipv6/ipv4 */
+    ASSERT_TRUE(aws_string_eq_c_str(ip_address, "::1") || aws_string_eq_c_str(ip_address, "127.0.0.1"));
+    int response_status = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_response_status_code(metrics, &response_status));
+    ASSERT_UINT_EQUALS(response_status, 200);
+    uint32_t stream_id = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_request_stream_id(metrics, &stream_id));
+    ASSERT_UINT_EQUALS(stream_id, 1);
+    const struct aws_string *request_path_query = NULL;
+    aws_s3_request_metrics_get_request_path_query(metrics, &request_path_query);
+    ASSERT_TRUE(request_path_query->len > 0);
+    const struct aws_string *host_address = NULL;
+    aws_s3_request_metrics_get_host_address(metrics, &host_address);
+    ASSERT_TRUE(host_address->len > 0);
+    aws_thread_id_t thread_id = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_thread_id(metrics, &thread_id));
+    size_t connection_id = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_connection_id(metrics, &connection_id));
+    ASSERT_UINT_EQUALS(aws_s3_request_metrics_get_error_code(metrics), AWS_ERROR_SUCCESS);
+    /* Get all those time stamp */
+    uint64_t time_stamp = 0;
+    aws_s3_request_metrics_get_start_timestamp_ns(metrics, &time_stamp);
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    aws_s3_request_metrics_get_end_timestamp_ns(metrics, &time_stamp);
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    aws_s3_request_metrics_get_total_duration_ns(metrics, &time_stamp);
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_send_start_timestamp_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_send_end_timestamp_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_sending_duration_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_receive_start_timestamp_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_receive_end_timestamp_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_receiving_duration_ns(metrics, &time_stamp));
+    ASSERT_FALSE(time_stamp == 0);
+    time_stamp = 0;
+    enum aws_s3_request_type request_type = 0;
+    aws_s3_request_metrics_get_request_type(metrics, &request_type);
+    ASSERT_UINT_EQUALS(AWS_S3_REQUEST_TYPE_CREATE_MULTIPART_UPLOAD, request_type);
+
+    /* Second metrics should be the Upload Part */
+    aws_array_list_get_at(metrics_list, (void **)&metrics, 1);
+    struct aws_byte_cursor header_value;
+    AWS_ZERO_STRUCT(header_value);
+    response_headers = NULL;
+    ASSERT_SUCCESS(aws_s3_request_metrics_get_response_headers(metrics, &response_headers));
+    ASSERT_SUCCESS(aws_http_headers_get(response_headers, aws_byte_cursor_from_c_str("ETag"), &header_value));
+    ASSERT_TRUE(aws_byte_cursor_eq_c_str(&header_value, "b54357faf0632cce46e942fa68356b38"));
+    ASSERT_SUCCESS(aws_http_headers_get(response_headers, aws_byte_cursor_from_c_str("Connection"), &header_value));
+    ASSERT_TRUE(aws_byte_cursor_eq_c_str(&header_value, "keep-alive"));
+    request_type = 0;
+    aws_s3_request_metrics_get_request_type(metrics, &request_type);
+    ASSERT_UINT_EQUALS(AWS_S3_REQUEST_TYPE_UPLOAD_PART, request_type);
+
+    /* Third metrics still be Upload Part */
+    aws_array_list_get_at(metrics_list, (void **)&metrics, 2);
+    request_type = 0;
+    aws_s3_request_metrics_get_request_type(metrics, &request_type);
+    ASSERT_UINT_EQUALS(AWS_S3_REQUEST_TYPE_UPLOAD_PART, request_type);
+
+    /* Fourth should be complete MPU */
+    aws_array_list_get_at(metrics_list, (void **)&metrics, 3);
+    request_type = 0;
+    aws_s3_request_metrics_get_request_type(metrics, &request_type);
+    ASSERT_UINT_EQUALS(AWS_S3_REQUEST_TYPE_COMPLETE_MULTIPART_UPLOAD, request_type);
+    /* All the rest should be similar */
+
+    return AWS_OP_SUCCESS;
+}
+
 TEST_CASE(multipart_upload_mock_server) {
     (void)ctx;
 
@@ -33,6 +130,48 @@ TEST_CASE(multipart_upload_mock_server) {
 
     struct aws_byte_cursor object_path = aws_byte_cursor_from_c_str("/default");
 
+    struct aws_s3_tester_meta_request_options put_options = {
+        .allocator = allocator,
+        .meta_request_type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT,
+        .client = client,
+        .checksum_algorithm = AWS_SCA_CRC32,
+        .validate_get_response_checksum = false,
+        .put_options =
+            {
+                .object_size_mb = 10,
+                .object_path_override = object_path,
+            },
+        .mock_server = true,
+    };
+    struct aws_s3_meta_request_test_results out_results;
+    aws_s3_meta_request_test_results_init(&out_results, allocator);
+    ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &put_options, &out_results));
+    ASSERT_SUCCESS(s_validate_mpu_mock_server_metrics(&out_results.synced_data.metrics));
+    aws_s3_meta_request_test_results_clean_up(&out_results);
+    aws_s3_client_release(client);
+    aws_s3_tester_clean_up(&tester);
+
+    return AWS_OP_SUCCESS;
+}
+
+TEST_CASE(multipart_upload_checksum_with_retry_mock_server) {
+    (void)ctx;
+    /**
+     * We had a memory leak when the retry was triggered and the checksum was calculated.
+     * The retry will initialize the checksum buffer again, but the previous one was not freed.
+     */
+    struct aws_s3_tester tester;
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
+    struct aws_s3_tester_client_options client_options = {
+        .part_size = MB_TO_BYTES(5),
+        .tls_usage = AWS_S3_TLS_DISABLED,
+    };
+
+    struct aws_s3_client *client = NULL;
+    ASSERT_SUCCESS(aws_s3_tester_client_new(&tester, &client_options, &client));
+
+    struct aws_byte_cursor object_path = aws_byte_cursor_from_c_str("/throttle");
+
     struct aws_s3_tester_meta_request_options put_options = {
         .allocator = allocator,
         .meta_request_type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT,
@@ -392,21 +531,6 @@ TEST_CASE(upload_part_invalid_response_mock_server) {
     return AWS_OP_SUCCESS;
 }
 
-static void s_resume_meta_request_progress(
-    struct aws_s3_meta_request *meta_request,
-    const struct aws_s3_meta_request_progress *progress,
-    void *user_data) {
-
-    (void)meta_request;
-    AWS_ASSERT(meta_request);
-    AWS_ASSERT(progress);
-    AWS_ASSERT(user_data);
-
-    struct aws_s3_meta_request_test_results *out_results = user_data;
-
-    aws_atomic_fetch_add(&out_results->total_bytes_uploaded, (size_t)progress->bytes_transferred);
-}
-
 /* Fake a MPU with 4 parts and the 2nd and 3rd have already completed and resume works fine */
 TEST_CASE(resume_first_part_not_completed_mock_server) {
     (void)ctx;
@@ -448,7 +572,6 @@ TEST_CASE(resume_first_part_not_completed_mock_server) {
     };
     struct aws_s3_meta_request_test_results out_results;
     aws_s3_meta_request_test_results_init(&out_results, allocator);
-    out_results.progress_callback = s_resume_meta_request_progress;
 
     ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &put_options, &out_results));
     /* Make Sure we only uploaded 2 parts. */
@@ -506,7 +629,6 @@ TEST_CASE(resume_mutli_page_list_parts_mock_server) {
     };
     struct aws_s3_meta_request_test_results out_results;
     aws_s3_meta_request_test_results_init(&out_results, allocator);
-    out_results.progress_callback = s_resume_meta_request_progress;
 
     ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &put_options, &out_results));
     /* Make Sure we only uploaded 2 parts. */
@@ -615,7 +737,6 @@ TEST_CASE(resume_after_finished_mock_server) {
     };
     struct aws_s3_meta_request_test_results out_results;
     aws_s3_meta_request_test_results_init(&out_results, allocator);
-    out_results.progress_callback = s_resume_meta_request_progress;
 
     ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &put_options, &out_results));
     /* The error code should be success, but there are no headers and stuff as no request was made. */
