@@ -182,6 +182,9 @@ typedef struct {
 // crypto_gcm_clmul_enabled returns one if the CLMUL implementation of GCM is
 // used.
 int crypto_gcm_clmul_enabled(void);
+// crypto_gcm_avx512_enabled returns one if the AVX512 VAES + VPCLMULQDQ
+// implementation of GCM is used.
+int crypto_gcm_avx512_enabled(void);
 #endif
 
 // CRYPTO_ghash_init writes a precomputed table of powers of |gcm_key| to
@@ -283,13 +286,24 @@ void gcm_init_avx(u128 Htable[16], const uint64_t Xi[2]);
 void gcm_gmult_avx(uint64_t Xi[2], const u128 Htable[16]);
 void gcm_ghash_avx(uint64_t Xi[2], const u128 Htable[16], const uint8_t *in,
                    size_t len);
-
+void gcm_init_avx512(u128 Htable[16], const uint64_t Xi[2]);
+void gcm_gmult_avx512(uint64_t Xi[2], const u128 Htable[16]);
+void gcm_ghash_avx512(uint64_t Xi[2], const u128 Htable[16], const uint8_t *in,
+                      size_t len);
 #define HW_GCM
 size_t aesni_gcm_encrypt(const uint8_t *in, uint8_t *out, size_t len,
                          const AES_KEY *key, uint8_t ivec[16], uint64_t *Xi);
 size_t aesni_gcm_decrypt(const uint8_t *in, uint8_t *out, size_t len,
                          const AES_KEY *key, uint8_t ivec[16], uint64_t *Xi);
-#endif  // OPENSSL_X86_64
+void gcm_setiv_avx512(const AES_KEY *key, const GCM128_CONTEXT *ctx,
+                      const uint8_t *iv, size_t ivlen);
+void aes_gcm_encrypt_avx512(const AES_KEY *key, const GCM128_CONTEXT *ctx,
+                            unsigned *pblocklen, const uint8_t *in, size_t len,
+                            uint8_t *out);
+void aes_gcm_decrypt_avx512(const AES_KEY *key, const GCM128_CONTEXT *ctx,
+                            unsigned *pblocklen, const uint8_t *in, size_t len,
+                            uint8_t *out);
+#endif  // OPENSSL_X86_64 && !MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX
 
 #if defined(OPENSSL_X86)
 #define GHASH_ASM_X86
@@ -318,11 +332,30 @@ void gcm_ghash_neon(uint64_t Xi[2], const u128 Htable[16], const uint8_t *inp,
 
 #if defined(OPENSSL_AARCH64)
 #define HW_GCM
+// Note that in the argument list of the following functions,
+// - the length is provided in bits (not bytes)
+// - the order of arguments is different from that of |aesni_gcm_encrypt|.
+
 // These functions are defined in aesv8-gcm-armv8.pl.
 void aes_gcm_enc_kernel(const uint8_t *in, uint64_t in_bits, void *out,
                         void *Xi, uint8_t *ivec, const AES_KEY *key);
 void aes_gcm_dec_kernel(const uint8_t *in, uint64_t in_bits, void *out,
                         void *Xi, uint8_t *ivec, const AES_KEY *key);
+
+// These functions are defined in aesv8-gcm-armv8-unroll8.pl.
+// They take input length in BITS and return number of BYTES processed.
+size_t aesv8_gcm_8x_enc_128(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
+size_t aesv8_gcm_8x_dec_128(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
+size_t aesv8_gcm_8x_enc_192(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
+size_t aesv8_gcm_8x_dec_192(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
+size_t aesv8_gcm_8x_enc_256(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
+size_t aesv8_gcm_8x_dec_256(const uint8_t *in, size_t bit_len, uint8_t *out,
+                            uint64_t *Xi, uint8_t ivec[16], const AES_KEY *key);
 #endif
 
 #elif defined(OPENSSL_PPC64LE)
