@@ -1,4 +1,3 @@
-/* crypto/x509/x509_lu.c */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -141,49 +140,25 @@ int X509_LOOKUP_by_subject(X509_LOOKUP *ctx, int type, X509_NAME *name,
   return ctx->method->get_by_subject(ctx, type, name, ret) > 0;
 }
 
-int X509_LOOKUP_by_issuer_serial(X509_LOOKUP *ctx, int type, X509_NAME *name,
-                                 ASN1_INTEGER *serial, X509_OBJECT *ret) {
-  if ((ctx->method == NULL) || (ctx->method->get_by_issuer_serial == NULL)) {
-    return 0;
-  }
-  return ctx->method->get_by_issuer_serial(ctx, type, name, serial, ret) > 0;
-}
-
-int X509_LOOKUP_by_fingerprint(X509_LOOKUP *ctx, int type, unsigned char *bytes,
-                               int len, X509_OBJECT *ret) {
-  if ((ctx->method == NULL) || (ctx->method->get_by_fingerprint == NULL)) {
-    return 0;
-  }
-  return ctx->method->get_by_fingerprint(ctx, type, bytes, len, ret) > 0;
-}
-
-int X509_LOOKUP_by_alias(X509_LOOKUP *ctx, int type, char *str, int len,
-                         X509_OBJECT *ret) {
-  if ((ctx->method == NULL) || (ctx->method->get_by_alias == NULL)) {
-    return 0;
-  }
-  return ctx->method->get_by_alias(ctx, type, str, len, ret) > 0;
-}
-
-static int x509_object_cmp(const X509_OBJECT **a, const X509_OBJECT **b) {
-  int ret;
-
-  ret = ((*a)->type - (*b)->type);
+static int x509_object_cmp(const X509_OBJECT *a, const X509_OBJECT *b) {
+  int ret = a->type - b->type;
   if (ret) {
     return ret;
   }
-  switch ((*a)->type) {
+  switch (a->type) {
     case X509_LU_X509:
-      ret = X509_subject_name_cmp((*a)->data.x509, (*b)->data.x509);
-      break;
+      return X509_subject_name_cmp(a->data.x509, b->data.x509);
     case X509_LU_CRL:
-      ret = X509_CRL_cmp((*a)->data.crl, (*b)->data.crl);
-      break;
+      return X509_CRL_cmp(a->data.crl, b->data.crl);
     default:
-      /* abort(); */
+      // abort();
       return 0;
   }
-  return ret;
+}
+
+static int x509_object_cmp_sk(const X509_OBJECT *const *a,
+                              const X509_OBJECT *const *b) {
+  return x509_object_cmp(*a, *b);
 }
 
 X509_STORE *X509_STORE_new(void) {
@@ -194,7 +169,7 @@ X509_STORE *X509_STORE_new(void) {
   }
   OPENSSL_memset(ret, 0, sizeof(*ret));
   CRYPTO_MUTEX_init(&ret->objs_lock);
-  ret->objs = sk_X509_OBJECT_new(x509_object_cmp);
+  ret->objs = sk_X509_OBJECT_new(x509_object_cmp_sk);
   if (ret->objs == NULL) {
     goto err;
   }
@@ -241,7 +216,7 @@ static void cleanup(X509_OBJECT *a) {
   } else if (a->type == X509_LU_CRL) {
     X509_CRL_free(a->data.crl);
   } else {
-    /* abort(); */
+    // abort();
   }
 
   OPENSSL_free(a);
@@ -289,7 +264,7 @@ X509_LOOKUP *X509_STORE_add_lookup(X509_STORE *v, X509_LOOKUP_METHOD *m) {
       return lu;
     }
   }
-  /* a new one */
+  // a new one
   lu = X509_LOOKUP_new(m);
   if (lu == NULL) {
     return NULL;
@@ -328,9 +303,7 @@ int X509_STORE_get_by_subject(X509_STORE_CTX *vs, int type, X509_NAME *name,
     }
   }
 
-  /*
-   * if (ret->data.ptr != NULL) X509_OBJECT_free_contents(ret);
-   */
+  // if (ret->data.ptr != NULL) X509_OBJECT_free_contents(ret);
 
   ret->type = tmp->type;
   ret->data.ptr = tmp->data.ptr;
@@ -347,7 +320,6 @@ static int x509_store_add(X509_STORE *ctx, void *x, int is_crl) {
 
   X509_OBJECT *const obj = (X509_OBJECT *)OPENSSL_malloc(sizeof(X509_OBJECT));
   if (obj == NULL) {
-    OPENSSL_PUT_ERROR(X509, ERR_R_MALLOC_FAILURE);
     return 0;
   }
 
@@ -364,7 +336,7 @@ static int x509_store_add(X509_STORE *ctx, void *x, int is_crl) {
 
   int ret = 1;
   int added = 0;
-  /* Duplicates are silently ignored */
+  // Duplicates are silently ignored
   if (!X509_OBJECT_retrieve_match(ctx->objs, obj)) {
     ret = added = (sk_X509_OBJECT_push(ctx->objs, obj) != 0);
   }
@@ -419,6 +391,13 @@ X509 *X509_OBJECT_get0_X509(const X509_OBJECT *a) {
   return a->data.x509;
 }
 
+X509_CRL *X509_OBJECT_get0_X509_CRL(const X509_OBJECT *a) {
+  if (a == NULL || a->type != X509_LU_CRL) {
+    return NULL;
+  }
+  return a->data.crl;
+}
+
 static int x509_object_idx_cnt(STACK_OF(X509_OBJECT) *h, int type,
                                X509_NAME *name, int *pnmatch) {
   X509_OBJECT stmp;
@@ -440,7 +419,7 @@ static int x509_object_idx_cnt(STACK_OF(X509_OBJECT) *h, int type,
       crl_info_s.issuer = name;
       break;
     default:
-      /* abort(); */
+      // abort();
       return -1;
   }
 
@@ -452,12 +431,10 @@ static int x509_object_idx_cnt(STACK_OF(X509_OBJECT) *h, int type,
 
   if (pnmatch != NULL) {
     int tidx;
-    const X509_OBJECT *tobj, *pstmp;
     *pnmatch = 1;
-    pstmp = &stmp;
     for (tidx = idx + 1; tidx < (int)sk_X509_OBJECT_num(h); tidx++) {
-      tobj = sk_X509_OBJECT_value(h, tidx);
-      if (x509_object_cmp(&tobj, &pstmp)) {
+      const X509_OBJECT *tobj = sk_X509_OBJECT_value(h, tidx);
+      if (x509_object_cmp(tobj, &stmp)) {
         break;
       }
       (*pnmatch)++;
@@ -498,10 +475,8 @@ STACK_OF(X509) *X509_STORE_get1_certs(X509_STORE_CTX *ctx, X509_NAME *nm) {
   CRYPTO_MUTEX_lock_write(&ctx->ctx->objs_lock);
   idx = x509_object_idx_cnt(ctx->ctx->objs, X509_LU_X509, nm, &cnt);
   if (idx < 0) {
-    /*
-     * Nothing found in cache: do lookup to possibly add new objects to
-     * cache
-     */
+    // Nothing found in cache: do lookup to possibly add new objects to
+    // cache
     X509_OBJECT xobj;
     CRYPTO_MUTEX_unlock_write(&ctx->ctx->objs_lock);
     if (!X509_STORE_get_by_subject(ctx, X509_LU_X509, nm, &xobj)) {
@@ -541,7 +516,7 @@ STACK_OF(X509_CRL) *X509_STORE_get1_crls(X509_STORE_CTX *ctx, X509_NAME *nm) {
     return NULL;
   }
 
-  /* Always do lookup to possibly add new CRLs to cache. */
+  // Always do lookup to possibly add new CRLs to cache.
   if (!X509_STORE_get_by_subject(ctx, X509_LU_CRL, nm, &xobj)) {
     sk_X509_CRL_free(sk);
     return NULL;
@@ -572,19 +547,17 @@ STACK_OF(X509_CRL) *X509_STORE_get1_crls(X509_STORE_CTX *ctx, X509_NAME *nm) {
 
 X509_OBJECT *X509_OBJECT_retrieve_match(STACK_OF(X509_OBJECT) *h,
                                         X509_OBJECT *x) {
-  size_t idx, i;
-  X509_OBJECT *obj;
-
   sk_X509_OBJECT_sort(h);
+  size_t idx;
   if (!sk_X509_OBJECT_find(h, &idx, x)) {
     return NULL;
   }
   if ((x->type != X509_LU_X509) && (x->type != X509_LU_CRL)) {
     return sk_X509_OBJECT_value(h, idx);
   }
-  for (i = idx; i < sk_X509_OBJECT_num(h); i++) {
-    obj = sk_X509_OBJECT_value(h, i);
-    if (x509_object_cmp((const X509_OBJECT **)&obj, (const X509_OBJECT **)&x)) {
+  for (size_t i = idx; i < sk_X509_OBJECT_num(h); i++) {
+    X509_OBJECT *obj = sk_X509_OBJECT_value(h, i);
+    if (x509_object_cmp(obj, x)) {
       return NULL;
     }
     if (x->type == X509_LU_X509) {
@@ -602,13 +575,11 @@ X509_OBJECT *X509_OBJECT_retrieve_match(STACK_OF(X509_OBJECT) *h,
   return NULL;
 }
 
-/*
- * Try to get issuer certificate from store. Due to limitations of the API
- * this can only retrieve a single certificate matching a given subject name.
- * However it will fill the cache with all matching certificates, so we can
- * examine the cache for all matches. Return values are: 1 lookup
- * successful.  0 certificate not found. -1 some other error.
- */
+// Try to get issuer certificate from store. Due to limitations of the API
+// this can only retrieve a single certificate matching a given subject name.
+// However it will fill the cache with all matching certificates, so we can
+// examine the cache for all matches. Return values are: 1 lookup
+// successful.  0 certificate not found. -1 some other error.
 int X509_STORE_CTX_get1_issuer(X509 **issuer, X509_STORE_CTX *ctx, X509 *x) {
   X509_NAME *xn;
   X509_OBJECT obj, *pobj;
@@ -618,23 +589,23 @@ int X509_STORE_CTX_get1_issuer(X509 **issuer, X509_STORE_CTX *ctx, X509 *x) {
   if (!X509_STORE_get_by_subject(ctx, X509_LU_X509, xn, &obj)) {
     return 0;
   }
-  /* If certificate matches all OK */
+  // If certificate matches all OK
   if (ctx->check_issued(ctx, x, obj.data.x509)) {
     *issuer = obj.data.x509;
     return 1;
   }
   X509_OBJECT_free_contents(&obj);
 
-  /* Else find index of first cert accepted by 'check_issued' */
+  // Else find index of first cert accepted by 'check_issued'
   ret = 0;
   CRYPTO_MUTEX_lock_write(&ctx->ctx->objs_lock);
   idx = X509_OBJECT_idx_by_subject(ctx->ctx->objs, X509_LU_X509, xn);
-  if (idx != -1) { /* should be true as we've had at least one
-                    * match */
-    /* Look through all matching certs for suitable issuer */
+  if (idx != -1) {  // should be true as we've had at least one
+                    // match
+    // Look through all matching certs for suitable issuer
     for (i = idx; i < sk_X509_OBJECT_num(ctx->ctx->objs); i++) {
       pobj = sk_X509_OBJECT_value(ctx->ctx->objs, i);
-      /* See if we've run past the matches */
+      // See if we've run past the matches
       if (pobj->type != X509_LU_X509) {
         break;
       }
