@@ -13,19 +13,20 @@
  * permissions and limitations under the License.
  */
 
+#include "error/s2n_errno.h"
+
 #include <errno.h>
-#include <strings.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "error/s2n_errno.h"
+#include <strings.h>
 
 #include "api/s2n.h"
 #include "utils/s2n_map.h"
 #include "utils/s2n_safety.h"
 
 #ifdef S2N_STACKTRACE
-#   include <execinfo.h>
+    #include <execinfo.h>
 #endif
 
 __thread int s2n_errno;
@@ -137,6 +138,7 @@ static const char *no_such_error = "Internal s2n error";
     ERR_ENTRY(S2N_ERR_HASH_INVALID_ALGORITHM, "invalid hash algorithm") \
     ERR_ENTRY(S2N_ERR_PRF_INVALID_ALGORITHM, "invalid prf hash algorithm") \
     ERR_ENTRY(S2N_ERR_PRF_INVALID_SEED, "invalid prf seeds provided") \
+    ERR_ENTRY(S2N_ERR_PRF_DERIVE, "error deriving a secret from the PRF") \
     ERR_ENTRY(S2N_ERR_P_HASH_INVALID_ALGORITHM, "invalid p_hash algorithm") \
     ERR_ENTRY(S2N_ERR_P_HASH_INIT_FAILED, "error initializing p_hash") \
     ERR_ENTRY(S2N_ERR_P_HASH_UPDATE_FAILED, "error updating p_hash") \
@@ -144,6 +146,7 @@ static const char *no_such_error = "Internal s2n error";
     ERR_ENTRY(S2N_ERR_P_HASH_WIPE_FAILED, "error wiping p_hash") \
     ERR_ENTRY(S2N_ERR_HMAC_INVALID_ALGORITHM, "invalid HMAC algorithm") \
     ERR_ENTRY(S2N_ERR_HKDF_OUTPUT_SIZE, "invalid HKDF output size") \
+    ERR_ENTRY(S2N_ERR_HKDF, "error generating HKDF output") \
     ERR_ENTRY(S2N_ERR_ALERT_PRESENT, "TLS alert is already pending") \
     ERR_ENTRY(S2N_ERR_HANDSHAKE_STATE, "Invalid handshake state encountered") \
     ERR_ENTRY(S2N_ERR_SHUTDOWN_PAUSED, "s2n_shutdown() called while paused") \
@@ -236,7 +239,7 @@ static const char *no_such_error = "Internal s2n error";
     ERR_ENTRY(S2N_ERR_SESSION_TICKET_NOT_SUPPORTED, "Session ticket not supported for this connection") \
     ERR_ENTRY(S2N_ERR_OCSP_NOT_SUPPORTED, "OCSP stapling was requested, but is not supported") \
     ERR_ENTRY(S2N_ERR_INVALID_SIGNATURE_ALGORITHMS_PREFERENCES, "Invalid signature algorithms preferences version") \
-    ERR_ENTRY(S2N_RSA_PSS_NOT_SUPPORTED, "RSA-PSS signing not supported by underlying libcrypto implementation") \
+    ERR_ENTRY(S2N_ERR_RSA_PSS_NOT_SUPPORTED, "RSA-PSS signing not supported by underlying libcrypto implementation") \
     ERR_ENTRY(S2N_ERR_MAX_INNER_PLAINTEXT_SIZE, "Inner plaintext size exceeds limit") \
     ERR_ENTRY(S2N_ERR_INVALID_ECC_PREFERENCES, "Invalid ecc curves preferences version") \
     ERR_ENTRY(S2N_ERR_RECORD_STUFFER_SIZE, "Record stuffer out of space") \
@@ -287,10 +290,15 @@ static const char *no_such_error = "Internal s2n error";
     ERR_ENTRY(S2N_ERR_INTERNAL_LIBCRYPTO_ERROR, "An internal error has occurred in the libcrypto API") \
     ERR_ENTRY(S2N_ERR_NO_RENEGOTIATION, "Only secure, server-initiated renegotiation is supported") \
     ERR_ENTRY(S2N_ERR_APP_DATA_BLOCKED, "Blocked on application data during handshake") \
-/* clang-format on */
+    ERR_ENTRY(S2N_ERR_ATOMIC, "Atomic operations in this environment would require locking") \
+    /* clang-format on */
 
-#define ERR_STR_CASE(ERR, str) case ERR: return str;
-#define ERR_NAME_CASE(ERR, str) case ERR: return #ERR;
+#define ERR_STR_CASE(ERR, str) \
+    case ERR:                  \
+        return str;
+#define ERR_NAME_CASE(ERR, str) \
+    case ERR:                   \
+        return #ERR;
 
 const char *s2n_strerror(int error, const char *lang)
 {
@@ -317,7 +325,7 @@ const char *s2n_strerror(int error, const char *lang)
         case S2N_ERR_T_USAGE_END:
             break;
 
-        /* No default to make compiler fail on missing values */
+            /* No default to make compiler fail on missing values */
     }
 
     return no_such_error;
@@ -340,7 +348,7 @@ const char *s2n_strerror_name(int error)
         case S2N_ERR_T_USAGE_END:
             break;
 
-        /* No default to make compiler fail on missing values */
+            /* No default to make compiler fail on missing values */
     }
 
     return no_such_error;
@@ -369,7 +377,6 @@ int s2n_error_get_type(int error)
     return (error >> S2N_ERR_NUM_VALUE_BITS);
 }
 
-
 /* https://www.gnu.org/software/libc/manual/html_node/Backtraces.html */
 static bool s_s2n_stack_traces_enabled = false;
 
@@ -386,15 +393,15 @@ int s2n_stack_traces_enabled_set(bool newval)
 
 #ifdef S2N_STACKTRACE
 
-#define MAX_BACKTRACE_DEPTH 20
-__thread struct s2n_stacktrace tl_stacktrace = {0};
+    #define MAX_BACKTRACE_DEPTH 20
+__thread struct s2n_stacktrace tl_stacktrace = { 0 };
 
 int s2n_free_stacktrace(void)
 {
     if (tl_stacktrace.trace != NULL) {
         free(tl_stacktrace.trace);
-	struct s2n_stacktrace zero_stacktrace = {0};
-	tl_stacktrace = zero_stacktrace;
+        struct s2n_stacktrace zero_stacktrace = { 0 };
+        tl_stacktrace = zero_stacktrace;
     }
     return S2N_SUCCESS;
 }
@@ -414,7 +421,8 @@ int s2n_calculate_stacktrace(void)
     return S2N_SUCCESS;
 }
 
-int s2n_get_stacktrace(struct s2n_stacktrace *trace) {
+int s2n_get_stacktrace(struct s2n_stacktrace *trace)
+{
     *trace = tl_stacktrace;
     return S2N_SUCCESS;
 }
@@ -422,20 +430,20 @@ int s2n_get_stacktrace(struct s2n_stacktrace *trace) {
 int s2n_print_stacktrace(FILE *fptr)
 {
     if (!s_s2n_stack_traces_enabled) {
-      fprintf(fptr, "%s\n%s\n",
-	      "NOTE: Some details are omitted, run with S2N_PRINT_STACKTRACE=1 for a verbose backtrace.",
-	      "See https://github.com/aws/s2n-tls/blob/main/docs/USAGE-GUIDE.md");
+        fprintf(fptr, "%s\n%s\n",
+                "NOTE: Some details are omitted, run with S2N_PRINT_STACKTRACE=1 for a verbose backtrace.",
+                "See https://github.com/aws/s2n-tls/blob/main/docs/USAGE-GUIDE.md");
         return S2N_SUCCESS;
     }
 
     fprintf(fptr, "\nStacktrace is:\n");
-    for (int i = 0; i < tl_stacktrace.trace_size; ++i){
-        fprintf(fptr, "%s\n",  tl_stacktrace.trace[i]);
+    for (int i = 0; i < tl_stacktrace.trace_size; ++i) {
+        fprintf(fptr, "%s\n", tl_stacktrace.trace[i]);
     }
     return S2N_SUCCESS;
 }
 
-#else /* !S2N_STACKTRACE */
+#else  /* !S2N_STACKTRACE */
 int s2n_free_stacktrace(void)
 {
     S2N_ERROR(S2N_ERR_UNIMPLEMENTED);
@@ -443,8 +451,7 @@ int s2n_free_stacktrace(void)
 
 int s2n_calculate_stacktrace(void)
 {
-    if (!s_s2n_stack_traces_enabled)
-    {
+    if (!s_s2n_stack_traces_enabled) {
         return S2N_SUCCESS;
     }
 
