@@ -165,6 +165,7 @@ typedef enum {
     S2N_ERR_HASH_INVALID_ALGORITHM,
     S2N_ERR_PRF_INVALID_ALGORITHM,
     S2N_ERR_PRF_INVALID_SEED,
+    S2N_ERR_PRF_DERIVE,
     S2N_ERR_P_HASH_INVALID_ALGORITHM,
     S2N_ERR_P_HASH_INIT_FAILED,
     S2N_ERR_P_HASH_UPDATE_FAILED,
@@ -172,6 +173,7 @@ typedef enum {
     S2N_ERR_P_HASH_WIPE_FAILED,
     S2N_ERR_HMAC_INVALID_ALGORITHM,
     S2N_ERR_HKDF_OUTPUT_SIZE,
+    S2N_ERR_HKDF,
     S2N_ERR_ALERT_PRESENT,
     S2N_ERR_HANDSHAKE_STATE,
     S2N_ERR_SHUTDOWN_PAUSED,
@@ -278,7 +280,7 @@ typedef enum {
     S2N_ERR_SESSION_TICKET_NOT_SUPPORTED,
     S2N_ERR_OCSP_NOT_SUPPORTED,
     S2N_ERR_INVALID_SIGNATURE_ALGORITHMS_PREFERENCES,
-    S2N_RSA_PSS_NOT_SUPPORTED,
+    S2N_ERR_RSA_PSS_NOT_SUPPORTED,
     S2N_ERR_INVALID_ECC_PREFERENCES,
     S2N_ERR_INVALID_SECURITY_POLICY,
     S2N_ERR_INVALID_KEM_PREFERENCES,
@@ -304,6 +306,7 @@ typedef enum {
     S2N_ERR_SECRET_SCHEDULE_STATE,
     S2N_ERR_CERT_OWNERSHIP,
     S2N_ERR_INTERNAL_LIBCRYPTO_ERROR,
+    S2N_ERR_ATOMIC,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
@@ -355,10 +358,10 @@ struct s2n_stacktrace {
     int trace_size;
 };
 
-extern bool s2n_stack_traces_enabled();
-extern int s2n_stack_traces_enabled_set(bool newval);
+bool s2n_stack_traces_enabled();
+int s2n_stack_traces_enabled_set(bool newval);
 
-extern int s2n_calculate_stacktrace(void);
-extern int s2n_print_stacktrace(FILE *fptr);
-extern int s2n_free_stacktrace(void);
-extern int s2n_get_stacktrace(struct s2n_stacktrace *trace);
+int s2n_calculate_stacktrace(void);
+int s2n_print_stacktrace(FILE *fptr);
+int s2n_free_stacktrace(void);
+int s2n_get_stacktrace(struct s2n_stacktrace *trace);
