@@ -39,6 +39,7 @@
 #include "tls/s2n_security_policies.h"
 #include "tls/s2n_tls_parameters.h"
 #include "tls/s2n_x509_validator.h"
+#include "utils/s2n_atomic.h"
 #include "utils/s2n_mem.h"
 #include "utils/s2n_timer.h"
 
@@ -66,9 +67,6 @@ struct s2n_connection {
     /* Connection can be used by a QUIC implementation */
     unsigned quic_enabled : 1;
 
-    /* Determines if we're currently sending or receiving in s2n_shutdown */
-    unsigned close_notify_queued : 1;
-
     /* s2n does not support renegotiation.
      * RFC5746 Section 4.3 suggests servers implement a minimal version of the
      * renegotiation_info extension even if renegotiation is not supported.
@@ -113,9 +111,6 @@ struct s2n_connection {
      * even when setting a new config. */
     unsigned psk_mode_overridden : 1;
 
-    /* Have we received a close notify alert from the peer. */
-    unsigned close_notify_received : 1;
-
     /* Connection negotiated an EMS */
     unsigned ems_negotiated : 1;
 
@@ -138,6 +133,9 @@ struct s2n_connection {
     unsigned ktls_send_enabled : 1;
     unsigned ktls_recv_enabled : 1;
 
+    /* Indicates whether the connection should request OCSP stapling from the peer */
+    unsigned request_ocsp_status : 1;
+
     /* The configuration (cert, key .. etc ) */
     struct s2n_config *config;
 
@@ -198,6 +196,11 @@ struct s2n_connection {
     uint8_t client_protocol_version;
     uint8_t server_protocol_version;
     uint8_t actual_protocol_version;
+    /* The version stored in the ticket / session we are resuming.
+     * We expect the connection to negotiate this version during
+     * the resumption handshake.
+     */
+    uint8_t resume_protocol_version;
 
     /* Flag indicating whether a protocol version has been
      * negotiated yet. */
@@ -206,7 +209,7 @@ struct s2n_connection {
     /* Our crypto parameters */
     struct s2n_crypto_parameters *initial;
     struct s2n_crypto_parameters *secure;
-    union s2n_secrets secrets;
+    struct s2n_secrets secrets;
 
     /* Which set is the client/server actually using? */
     struct s2n_crypto_parameters *client;
@@ -259,18 +262,17 @@ struct s2n_connection {
     uint8_t alert_in_data[S2N_ALERT_LENGTH];
     struct s2n_stuffer alert_in;
 
-    /* An alert may be partially written in the outbound
-     * direction, so we keep this as a small 2 byte queue.
-     *
-     * We keep separate queues for alerts generated by
-     * readers (a response to an alert from a peer) and writers (an
-     * intentional shutdown) so that the s2n reader and writer
-     * can be separate duplex I/O threads.
+    /* Both readers and writers can trigger alerts.
+     * We prioritize writer alerts over reader alerts.
      */
-    uint8_t reader_alert_out_data[S2N_ALERT_LENGTH];
-    uint8_t writer_alert_out_data[S2N_ALERT_LENGTH];
-    struct s2n_stuffer reader_alert_out;
-    struct s2n_stuffer writer_alert_out;
+    uint8_t writer_alert_out;
+    uint8_t reader_alert_out;
+    uint8_t reader_warning_out;
+    bool alert_sent;
+
+    /* Receiving error or close_notify alerts changes the behavior of s2n_shutdown_send */
+    s2n_atomic_flag error_alert_received;
+    s2n_atomic_flag close_notify_received;
 
     /* Our handshake state machine */
     struct s2n_handshake handshake;
@@ -308,21 +310,11 @@ struct s2n_connection {
     uint64_t wire_bytes_out;
     uint64_t early_data_bytes;
 
-    /* Is the connection open or closed ? We use C's only
-     * atomic type as both the reader and the writer threads
-     * may declare a connection closed.
-     *
-     * A connection can be gracefully closed or hard-closed.
-     * When gracefully closed the reader or the writer mark
-     * the connection as closing, and then the writer will
-     * send an alert message before closing the connection
-     * and marking it as closed.
-     *
-     * A hard-close goes straight to closed with no alert
-     * message being sent.
+    /* Either the reader or the writer can trigger both sides of the connection
+     * to close in response to a fatal error.
      */
-    sig_atomic_t closing;
-    sig_atomic_t closed;
+    s2n_atomic_flag read_closed;
+    s2n_atomic_flag write_closed;
 
     /* TLS extension data */
     char server_name[S2N_MAX_SERVER_NAME + 1];
@@ -397,8 +389,14 @@ S2N_CLEANUP_RESULT s2n_connection_ptr_free(struct s2n_connection **s2n_connectio
 int s2n_connection_is_managed_corked(const struct s2n_connection *s2n_connection);
 int s2n_connection_is_client_auth_enabled(struct s2n_connection *s2n_connection);
 
-/* Kill a bad connection */
-int s2n_connection_kill(struct s2n_connection *conn);
+typedef enum {
+    S2N_IO_WRITABLE,
+    S2N_IO_READABLE,
+    S2N_IO_FULL_DUPLEX,
+    S2N_IO_CLOSED,
+} s2n_io_status;
+bool s2n_connection_check_io_status(struct s2n_connection *conn, s2n_io_status status);
+S2N_RESULT s2n_connection_set_closed(struct s2n_connection *conn);
 
 /* Send/recv a stuffer to/from a connection */
 int s2n_connection_send_stuffer(struct s2n_stuffer *stuffer, struct s2n_connection *conn, uint32_t len);
