@@ -1622,10 +1622,7 @@ int s2n_negotiate_impl(struct s2n_connection *conn, s2n_blocked_status *blocked)
         /* Flush any pending I/O or alert messages */
         POSIX_GUARD(s2n_flush(conn, blocked));
 
-        /* If the connection is closed, the handshake will never complete. */
-        if (conn->closed) {
-            POSIX_BAIL(S2N_ERR_CLOSED);
-        }
+        POSIX_ENSURE(s2n_connection_check_io_status(conn, S2N_IO_FULL_DUPLEX), S2N_ERR_CLOSED);
 
         /* If the handshake was paused, retry the current message */
         if (conn->handshake.paused) {
