@@ -52,6 +52,7 @@ int s2n_key_update_send(struct s2n_connection *conn, s2n_blocked_status *blocked
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->secure);
+    POSIX_ENSURE_GTE(conn->actual_protocol_version, S2N_TLS13);
 
     struct s2n_blob sequence_number = { 0 };
     if (conn->mode == S2N_CLIENT) {
@@ -80,7 +81,7 @@ int s2n_key_update_send(struct s2n_connection *conn, s2n_blocked_status *blocked
         POSIX_GUARD(s2n_key_update_write(&key_update_blob));
 
         /* Encrypt the message */
-        POSIX_GUARD(s2n_record_write(conn, TLS_HANDSHAKE, &key_update_blob));
+        POSIX_GUARD_RESULT(s2n_record_write(conn, TLS_HANDSHAKE, &key_update_blob));
 
         /* Update encryption key */
         POSIX_GUARD(s2n_update_application_traffic_keys(conn, conn->mode, SENDING));
