@@ -12,23 +12,22 @@
  * express or implied. See the License for the specific language governing
  * permissions and limitations under the License.
  */
+#include "tls/s2n_resume.h"
+
 #include <math.h>
 #include <sys/param.h>
 
 #include "api/s2n.h"
-
 #include "error/s2n_errno.h"
 #include "stuffer/s2n_stuffer.h"
-#include "utils/s2n_safety.h"
-#include "utils/s2n_blob.h"
-#include "utils/s2n_random.h"
-#include "utils/s2n_set.h"
-
 #include "tls/s2n_cipher_suites.h"
 #include "tls/s2n_connection.h"
-#include "tls/s2n_resume.h"
 #include "tls/s2n_crypto.h"
 #include "tls/s2n_tls.h"
+#include "utils/s2n_blob.h"
+#include "utils/s2n_random.h"
+#include "utils/s2n_safety.h"
+#include "utils/s2n_set.h"
 
 int s2n_allowed_to_cache_connection(struct s2n_connection *conn)
 {
@@ -44,6 +43,23 @@ int s2n_allowed_to_cache_connection(struct s2n_connection *conn)
     return config->use_session_cache;
 }
 
+/* If a protocol version is required before the actual_protocol_version
+ * is negotiated, we should fall back to resume_protocol_version if available.
+ *
+ * This covers the case where the application requests a ticket / session state
+ * before a NewSessionTicket message has been sent or received. Historically,
+ * in that case we return the ticket / session state already set for the connection.
+ * resume_protocol_version represents the protocol version of that existing ticket / state.
+ */
+static uint8_t s2n_resume_protocol_version(struct s2n_connection *conn)
+{
+    if (!IS_NEGOTIATED(conn) && conn->resume_protocol_version) {
+        return conn->resume_protocol_version;
+    } else {
+        return conn->actual_protocol_version;
+    }
+}
+
 static int s2n_tls12_serialize_resumption_state(struct s2n_connection *conn, struct s2n_stuffer *to)
 {
     POSIX_ENSURE_REF(to);
@@ -59,10 +75,10 @@ static int s2n_tls12_serialize_resumption_state(struct s2n_connection *conn, str
 
     /* Write the entry */
     POSIX_GUARD(s2n_stuffer_write_uint8(to, S2N_SERIALIZED_FORMAT_TLS12_V3));
-    POSIX_GUARD(s2n_stuffer_write_uint8(to, conn->actual_protocol_version));
+    POSIX_GUARD(s2n_stuffer_write_uint8(to, s2n_resume_protocol_version(conn)));
     POSIX_GUARD(s2n_stuffer_write_bytes(to, conn->secure->cipher_suite->iana_value, S2N_TLS_CIPHER_SUITE_LEN));
     POSIX_GUARD(s2n_stuffer_write_uint64(to, now));
-    POSIX_GUARD(s2n_stuffer_write_bytes(to, conn->secrets.tls12.master_secret, S2N_TLS_SECRET_LEN));
+    POSIX_GUARD(s2n_stuffer_write_bytes(to, conn->secrets.version.tls12.master_secret, S2N_TLS_SECRET_LEN));
     POSIX_GUARD(s2n_stuffer_write_uint8(to, conn->ems_negotiated));
 
     return S2N_SUCCESS;
@@ -127,7 +143,7 @@ static S2N_RESULT s2n_tls13_serialize_resumption_state(struct s2n_connection *co
 
 static S2N_RESULT s2n_serialize_resumption_state(struct s2n_connection *conn, struct s2n_stuffer *out)
 {
-    if(conn->actual_protocol_version < S2N_TLS13) {
+    if (s2n_resume_protocol_version(conn) < S2N_TLS13) {
         RESULT_GUARD_POSIX(s2n_tls12_serialize_resumption_state(conn, out));
     } else {
         RESULT_GUARD(s2n_tls13_serialize_resumption_state(conn, out));
@@ -159,7 +175,7 @@ static int s2n_tls12_deserialize_resumption_state(struct s2n_connection *conn, s
     S2N_ERROR_IF(then > now, S2N_ERR_INVALID_SERIALIZED_SESSION_STATE);
     S2N_ERROR_IF(now - then > conn->config->session_state_lifetime_in_nanos, S2N_ERR_INVALID_SERIALIZED_SESSION_STATE);
 
-    POSIX_GUARD(s2n_stuffer_read_bytes(from, conn->secrets.tls12.master_secret, S2N_TLS_SECRET_LEN));
+    POSIX_GUARD(s2n_stuffer_read_bytes(from, conn->secrets.version.tls12.master_secret, S2N_TLS_SECRET_LEN));
 
     if (s2n_stuffer_data_available(from)) {
         uint8_t ems_negotiated = 0;
@@ -194,17 +210,17 @@ static int s2n_tls12_deserialize_resumption_state(struct s2n_connection *conn, s
 static int s2n_client_serialize_resumption_state(struct s2n_connection *conn, struct s2n_stuffer *to)
 {
     /* Serialize session ticket */
-   if (conn->config->use_tickets && conn->client_ticket.size > 0) {
-       POSIX_GUARD(s2n_stuffer_write_uint8(to, S2N_STATE_WITH_SESSION_TICKET));
-       POSIX_GUARD(s2n_stuffer_write_uint16(to, conn->client_ticket.size));
-       POSIX_GUARD(s2n_stuffer_write(to, &conn->client_ticket));
-   } else {
-       /* Serialize session id */
-       POSIX_ENSURE_LT(conn->actual_protocol_version, S2N_TLS13);
-       POSIX_GUARD(s2n_stuffer_write_uint8(to, S2N_STATE_WITH_SESSION_ID));
-       POSIX_GUARD(s2n_stuffer_write_uint8(to, conn->session_id_len));
-       POSIX_GUARD(s2n_stuffer_write_bytes(to, conn->session_id, conn->session_id_len));
-   }
+    if (conn->config->use_tickets && conn->client_ticket.size > 0) {
+        POSIX_GUARD(s2n_stuffer_write_uint8(to, S2N_STATE_WITH_SESSION_TICKET));
+        POSIX_GUARD(s2n_stuffer_write_uint16(to, conn->client_ticket.size));
+        POSIX_GUARD(s2n_stuffer_write(to, &conn->client_ticket));
+    } else {
+        /* Serialize session id */
+        POSIX_ENSURE_LT(conn->actual_protocol_version, S2N_TLS13);
+        POSIX_GUARD(s2n_stuffer_write_uint8(to, S2N_STATE_WITH_SESSION_ID));
+        POSIX_GUARD(s2n_stuffer_write_uint8(to, conn->session_id_len));
+        POSIX_GUARD(s2n_stuffer_write_bytes(to, conn->session_id, conn->session_id_len));
+    }
 
     /* Serialize session state */
     POSIX_GUARD_RESULT(s2n_serialize_resumption_state(conn, to));
@@ -217,7 +233,7 @@ static S2N_RESULT s2n_tls12_client_deserialize_session_state(struct s2n_connecti
     RESULT_ENSURE_REF(conn);
     RESULT_ENSURE_REF(from);
 
-    RESULT_GUARD_POSIX(s2n_stuffer_read_uint8(from, &conn->actual_protocol_version));
+    RESULT_GUARD_POSIX(s2n_stuffer_read_uint8(from, &conn->resume_protocol_version));
 
     uint8_t *cipher_suite_wire = s2n_stuffer_raw_read(from, S2N_TLS_CIPHER_SUITE_LEN);
     RESULT_ENSURE_REF(cipher_suite_wire);
@@ -226,7 +242,7 @@ static S2N_RESULT s2n_tls12_client_deserialize_session_state(struct s2n_connecti
     uint64_t then = 0;
     RESULT_GUARD_POSIX(s2n_stuffer_read_uint64(from, &then));
 
-    RESULT_GUARD_POSIX(s2n_stuffer_read_bytes(from, conn->secrets.tls12.master_secret, S2N_TLS_SECRET_LEN));
+    RESULT_GUARD_POSIX(s2n_stuffer_read_bytes(from, conn->secrets.version.tls12.master_secret, S2N_TLS_SECRET_LEN));
 
     if (s2n_stuffer_data_available(from)) {
         uint8_t ems_negotiated = 0;
@@ -356,7 +372,7 @@ static int s2n_client_deserialize_with_session_id(struct s2n_connection *conn, s
     POSIX_GUARD(s2n_stuffer_read_uint8(from, &session_id_len));
 
     if (session_id_len == 0 || session_id_len > S2N_TLS_SESSION_ID_MAX_LEN
-        || session_id_len > s2n_stuffer_data_available(from)) {
+            || session_id_len > s2n_stuffer_data_available(from)) {
         POSIX_BAIL(S2N_ERR_INVALID_SERIALIZED_SESSION_STATE);
     }
 
@@ -391,14 +407,14 @@ static int s2n_client_deserialize_resumption_state(struct s2n_connection *conn,
     POSIX_GUARD(s2n_stuffer_read_uint8(from, &format));
 
     switch (format) {
-    case S2N_STATE_WITH_SESSION_ID:
-        POSIX_GUARD(s2n_client_deserialize_with_session_id(conn, from));
-        break;
-    case S2N_STATE_WITH_SESSION_TICKET:
-        POSIX_GUARD(s2n_client_deserialize_with_session_ticket(conn, from));
-        break;
-    default:
-        POSIX_BAIL(S2N_ERR_INVALID_SERIALIZED_SESSION_STATE);
+        case S2N_STATE_WITH_SESSION_ID:
+            POSIX_GUARD(s2n_client_deserialize_with_session_id(conn, from));
+            break;
+        case S2N_STATE_WITH_SESSION_TICKET:
+            POSIX_GUARD(s2n_client_deserialize_with_session_ticket(conn, from));
+            break;
+        default:
+            POSIX_BAIL(S2N_ERR_INVALID_SERIALIZED_SESSION_STATE);
     }
 
     return 0;
@@ -410,7 +426,7 @@ int s2n_resume_from_cache(struct s2n_connection *conn)
     S2N_ERROR_IF(conn->session_id_len > S2N_TLS_SESSION_ID_MAX_LEN, S2N_ERR_SESSION_ID_TOO_LONG);
 
     uint8_t data[S2N_TLS12_TICKET_SIZE_IN_BYTES] = { 0 };
-    struct s2n_blob entry = {0};
+    struct s2n_blob entry = { 0 };
     POSIX_GUARD(s2n_blob_init(&entry, data, S2N_TLS12_TICKET_SIZE_IN_BYTES));
     uint64_t size = entry.size;
     int result = conn->config->cache_retrieve(conn, conn->config->cache_retrieve_data, conn->session_id, conn->session_id_len, entry.data, &size);
@@ -421,7 +437,7 @@ int s2n_resume_from_cache(struct s2n_connection *conn)
 
     S2N_ERROR_IF(size != entry.size, S2N_ERR_SIZE_MISMATCH);
 
-    struct s2n_stuffer from = {0};
+    struct s2n_stuffer from = { 0 };
     POSIX_GUARD(s2n_stuffer_init(&from, &entry));
     POSIX_GUARD(s2n_stuffer_write(&from, &entry));
     POSIX_GUARD(s2n_decrypt_session_cache(conn, &from));
@@ -432,9 +448,9 @@ int s2n_resume_from_cache(struct s2n_connection *conn)
 S2N_RESULT s2n_store_to_cache(struct s2n_connection *conn)
 {
     uint8_t data[S2N_TLS12_TICKET_SIZE_IN_BYTES] = { 0 };
-    struct s2n_blob entry = {0};
+    struct s2n_blob entry = { 0 };
     RESULT_GUARD_POSIX(s2n_blob_init(&entry, data, S2N_TLS12_TICKET_SIZE_IN_BYTES));
-    struct s2n_stuffer to = {0};
+    struct s2n_stuffer to = { 0 };
 
     /* session_id_len should always be >0 since either the Client provided a SessionId or the Server generated a new
      * one for the Client */
@@ -455,11 +471,11 @@ int s2n_connection_set_session(struct s2n_connection *conn, const uint8_t *sessi
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(session);
 
-    DEFER_CLEANUP(struct s2n_blob session_data = {0}, s2n_free);
+    DEFER_CLEANUP(struct s2n_blob session_data = { 0 }, s2n_free);
     POSIX_GUARD(s2n_alloc(&session_data, length));
     POSIX_CHECKED_MEMCPY(session_data.data, session, length);
 
-    struct s2n_stuffer from = {0};
+    struct s2n_stuffer from = { 0 };
     POSIX_GUARD(s2n_stuffer_init(&from, &session_data));
     POSIX_GUARD(s2n_stuffer_write(&from, &session_data));
     POSIX_GUARD(s2n_client_deserialize_resumption_state(conn, &from));
@@ -471,19 +487,20 @@ int s2n_connection_get_session(struct s2n_connection *conn, uint8_t *session, si
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(session);
 
-    int len = s2n_connection_get_session_length(conn);
+    const int len = s2n_connection_get_session_length(conn);
+    POSIX_GUARD(len);
 
     if (len == 0) {
         return 0;
     }
 
-    S2N_ERROR_IF(len > max_length, S2N_ERR_SERIALIZED_SESSION_STATE_TOO_LONG);
+    POSIX_ENSURE((size_t) len <= max_length, S2N_ERR_SERIALIZED_SESSION_STATE_TOO_LONG);
 
-    struct s2n_blob serialized_data = {0};
+    struct s2n_blob serialized_data = { 0 };
     POSIX_GUARD(s2n_blob_init(&serialized_data, session, len));
     POSIX_GUARD(s2n_blob_zero(&serialized_data));
 
-    struct s2n_stuffer to = {0};
+    struct s2n_stuffer to = { 0 };
     POSIX_GUARD(s2n_stuffer_init(&to, &serialized_data));
     POSIX_GUARD(s2n_client_serialize_resumption_state(conn, &to));
 
@@ -505,7 +522,7 @@ S2N_RESULT s2n_connection_get_session_state_size(struct s2n_connection *conn, si
     RESULT_ENSURE_REF(conn->secure);
     RESULT_ENSURE_REF(state_size);
 
-    if (conn->actual_protocol_version < S2N_TLS13) {
+    if (s2n_resume_protocol_version(conn) < S2N_TLS13) {
         *state_size = S2N_TLS12_STATE_SIZE_IN_BYTES;
         return S2N_RESULT_OK;
     }
@@ -557,7 +574,7 @@ int s2n_connection_get_session_length(struct s2n_connection *conn)
 int s2n_connection_is_session_resumed(struct s2n_connection *conn)
 {
     return conn && IS_RESUMPTION_HANDSHAKE(conn)
-        && (conn->actual_protocol_version < S2N_TLS13 || conn->psk_params.type == S2N_PSK_TYPE_RESUMPTION);
+            && (conn->actual_protocol_version < S2N_TLS13 || conn->psk_params.type == S2N_PSK_TYPE_RESUMPTION);
 }
 
 int s2n_connection_is_ocsp_stapled(struct s2n_connection *conn)
@@ -583,7 +600,7 @@ int s2n_config_is_encrypt_decrypt_key_available(struct s2n_config *config)
 
     for (uint32_t i = ticket_keys_len; i > 0; i--) {
         uint32_t idx = i - 1;
-        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **)&ticket_key));
+        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **) &ticket_key));
         uint64_t key_intro_time = ticket_key->intro_timestamp;
 
         if (key_intro_time < now
@@ -600,9 +617,9 @@ int s2n_config_is_encrypt_decrypt_key_available(struct s2n_config *config)
  * Higher the weight of the key, higher the probability of being picked.
  */
 int s2n_compute_weight_of_encrypt_decrypt_keys(struct s2n_config *config,
-                                               uint8_t *encrypt_decrypt_keys_index,
-                                               uint8_t num_encrypt_decrypt_keys,
-                                               uint64_t now)
+        uint8_t *encrypt_decrypt_keys_index,
+        uint8_t num_encrypt_decrypt_keys,
+        uint64_t now)
 {
     double total_weight = 0;
     struct s2n_ticket_key_weight ticket_keys_weight[S2N_MAX_TICKET_KEYS];
@@ -610,7 +627,7 @@ int s2n_compute_weight_of_encrypt_decrypt_keys(struct s2n_config *config,
 
     /* Compute weight of encrypt-decrypt keys */
     for (int i = 0; i < num_encrypt_decrypt_keys; i++) {
-        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, encrypt_decrypt_keys_index[i], (void **)&ticket_key));
+        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, encrypt_decrypt_keys_index[i], (void **) &ticket_key));
 
         uint64_t key_intro_time = ticket_key->intro_timestamp;
         uint64_t key_encryption_peak_time = key_intro_time + (config->encrypt_decrypt_key_lifetime_in_nanos / 2);
@@ -630,7 +647,7 @@ int s2n_compute_weight_of_encrypt_decrypt_keys(struct s2n_config *config,
     /* Pick a random number in [0, 1). Using 53 bits (IEEE 754 double-precision floats). */
     uint64_t random_int = 0;
     POSIX_GUARD_RESULT(s2n_public_random(pow(2, 53), &random_int));
-    double random = (double)random_int / (double)pow(2, 53);
+    double random = (double) random_int / (double) pow(2, 53);
 
     /* Compute cumulative weight of encrypt-decrypt keys */
     for (int i = 0; i < num_encrypt_decrypt_keys; i++) {
@@ -666,7 +683,7 @@ struct s2n_ticket_key *s2n_get_ticket_encrypt_decrypt_key(struct s2n_config *con
 
     for (uint32_t i = ticket_keys_len; i > 0; i--) {
         uint32_t idx = i - 1;
-        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **)&ticket_key));
+        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **) &ticket_key));
         uint64_t key_intro_time = ticket_key->intro_timestamp;
 
         if (key_intro_time < now
@@ -681,14 +698,14 @@ struct s2n_ticket_key *s2n_get_ticket_encrypt_decrypt_key(struct s2n_config *con
     }
 
     if (num_encrypt_decrypt_keys == 1) {
-        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, encrypt_decrypt_keys_index[0], (void **)&ticket_key));
+        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, encrypt_decrypt_keys_index[0], (void **) &ticket_key));
         return ticket_key;
     }
 
     int8_t idx;
     PTR_GUARD_POSIX(idx = s2n_compute_weight_of_encrypt_decrypt_keys(config, encrypt_decrypt_keys_index, num_encrypt_decrypt_keys, now));
 
-    PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **)&ticket_key));
+    PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, idx, (void **) &ticket_key));
     return ticket_key;
 }
 
@@ -706,13 +723,13 @@ struct s2n_ticket_key *s2n_find_ticket_key(struct s2n_config *config, const uint
     PTR_GUARD_RESULT(s2n_set_len(config->ticket_keys, &ticket_keys_len));
 
     for (uint32_t i = 0; i < ticket_keys_len; i++) {
-        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, i, (void **)&ticket_key));
+        PTR_GUARD_RESULT(s2n_set_get(config->ticket_keys, i, (void **) &ticket_key));
 
         if (memcmp(ticket_key->key_name, name, S2N_TICKET_KEY_NAME_LEN) == 0) {
-
             /* Check to see if the key has expired */
-            if (now >= ticket_key->intro_timestamp +
-                                config->encrypt_decrypt_key_lifetime_in_nanos + config->decrypt_key_lifetime_in_nanos) {
+            if (now >= ticket_key->intro_timestamp
+                            + config->encrypt_decrypt_key_lifetime_in_nanos
+                            + config->decrypt_key_lifetime_in_nanos) {
                 s2n_config_wipe_expired_ticket_crypto_keys(config, i);
 
                 return NULL;
@@ -728,17 +745,17 @@ struct s2n_ticket_key *s2n_find_ticket_key(struct s2n_config *config, const uint
 int s2n_encrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *to)
 {
     struct s2n_ticket_key *key;
-    struct s2n_session_key aes_ticket_key = {0};
-    struct s2n_blob aes_key_blob = {0};
+    struct s2n_session_key aes_ticket_key = { 0 };
+    struct s2n_blob aes_key_blob = { 0 };
 
     uint8_t iv_data[S2N_TLS_GCM_IV_LEN] = { 0 };
-    struct s2n_blob iv = {0};
+    struct s2n_blob iv = { 0 };
     POSIX_GUARD(s2n_blob_init(&iv, iv_data, sizeof(iv_data)));
 
     uint8_t aad_data[S2N_TICKET_AAD_LEN] = { 0 };
-    struct s2n_blob aad_blob = {0};
+    struct s2n_blob aad_blob = { 0 };
     POSIX_GUARD(s2n_blob_init(&aad_blob, aad_data, sizeof(aad_data)));
-    struct s2n_stuffer aad = {0};
+    struct s2n_stuffer aad = { 0 };
 
     key = s2n_get_ticket_encrypt_decrypt_key(conn->config);
 
@@ -782,8 +799,8 @@ int s2n_encrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *
 int s2n_decrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *from)
 {
     struct s2n_ticket_key *key;
-    DEFER_CLEANUP(struct s2n_session_key aes_ticket_key = {0}, s2n_session_key_free);
-    struct s2n_blob aes_key_blob = {0};
+    DEFER_CLEANUP(struct s2n_session_key aes_ticket_key = { 0 }, s2n_session_key_free);
+    struct s2n_blob aes_key_blob = { 0 };
 
     uint8_t key_name[S2N_TICKET_KEY_NAME_LEN] = { 0 };
 
@@ -792,9 +809,9 @@ int s2n_decrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *
     POSIX_GUARD(s2n_blob_init(&iv, iv_data, sizeof(iv_data)));
 
     uint8_t aad_data[S2N_TICKET_AAD_LEN] = { 0 };
-    struct s2n_blob aad_blob = {0};
+    struct s2n_blob aad_blob = { 0 };
     POSIX_GUARD(s2n_blob_init(&aad_blob, aad_data, sizeof(aad_data)));
-    struct s2n_stuffer aad = {0};
+    struct s2n_stuffer aad = { 0 };
 
     POSIX_GUARD(s2n_stuffer_read_bytes(from, key_name, s2n_array_len(key_name)));
 
@@ -854,27 +871,27 @@ int s2n_encrypt_session_cache(struct s2n_connection *conn, struct s2n_stuffer *t
 int s2n_decrypt_session_cache(struct s2n_connection *conn, struct s2n_stuffer *from)
 {
     struct s2n_ticket_key *key;
-    struct s2n_session_key aes_ticket_key = {0};
-    struct s2n_blob aes_key_blob = {0};
+    struct s2n_session_key aes_ticket_key = { 0 };
+    struct s2n_blob aes_key_blob = { 0 };
 
     uint8_t key_name[S2N_TICKET_KEY_NAME_LEN] = { 0 };
 
     uint8_t iv_data[S2N_TLS_GCM_IV_LEN] = { 0 };
-    struct s2n_blob iv = {0};
+    struct s2n_blob iv = { 0 };
     POSIX_GUARD(s2n_blob_init(&iv, iv_data, sizeof(iv_data)));
 
     uint8_t aad_data[S2N_TICKET_AAD_LEN] = { 0 };
-    struct s2n_blob aad_blob = {0};
+    struct s2n_blob aad_blob = { 0 };
     POSIX_GUARD(s2n_blob_init(&aad_blob, aad_data, sizeof(aad_data)));
-    struct s2n_stuffer aad = {0};
+    struct s2n_stuffer aad = { 0 };
 
     uint8_t s_data[S2N_TLS12_STATE_SIZE_IN_BYTES] = { 0 };
-    struct s2n_blob state_blob = {0};
+    struct s2n_blob state_blob = { 0 };
     POSIX_GUARD(s2n_blob_init(&state_blob, s_data, sizeof(s_data)));
-    struct s2n_stuffer state = {0};
+    struct s2n_stuffer state = { 0 };
 
-    uint8_t en_data[S2N_TLS12_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN] = {0};
-    struct s2n_blob en_blob = {0};
+    uint8_t en_data[S2N_TLS12_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN] = { 0 };
+    struct s2n_blob en_blob = { 0 };
     POSIX_GUARD(s2n_blob_init(&en_blob, en_data, sizeof(en_data)));
 
     POSIX_GUARD(s2n_stuffer_read_bytes(from, key_name, s2n_array_len(key_name)));
@@ -931,9 +948,10 @@ int s2n_config_wipe_expired_ticket_crypto_keys(struct s2n_config *config, int8_t
     POSIX_GUARD_RESULT(s2n_set_len(config->ticket_keys, &ticket_keys_len));
 
     for (uint32_t i = 0; i < ticket_keys_len; i++) {
-        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, i, (void **)&ticket_key));
-        if (now >= ticket_key->intro_timestamp +
-                   config->encrypt_decrypt_key_lifetime_in_nanos + config->decrypt_key_lifetime_in_nanos) {
+        POSIX_GUARD_RESULT(s2n_set_get(config->ticket_keys, i, (void **) &ticket_key));
+        if (now >= ticket_key->intro_timestamp
+                        + config->encrypt_decrypt_key_lifetime_in_nanos
+                        + config->decrypt_key_lifetime_in_nanos) {
             expired_keys_index[num_of_expired_keys] = i;
             num_of_expired_keys++;
         }
@@ -947,7 +965,6 @@ end:
     return 0;
 }
 
-
 int s2n_config_store_ticket_key(struct s2n_config *config, struct s2n_ticket_key *key)
 {
     /* Keys are stored from oldest to newest */
