@@ -48,6 +48,10 @@ struct mqtt_connection_binding {
      * Lets us invoke callbacks on the python object without preventing the GC from cleaning it up. */
     PyObject *self_proxy;
 
+    /* To not run into a segfault calling on_close with the connection being freed before the callback
+     * can be invoked, we need to keep the PyCapsule alive. */
+    PyObject *self_capsule;
+
     PyObject *on_connect;
     PyObject *on_any_publish;
 
@@ -56,6 +60,10 @@ struct mqtt_connection_binding {
 };
 
 static void s_mqtt_python_connection_finish_destruction(struct mqtt_connection_binding *py_connection) {
+
+    /* Do not call the on_stopped callback if the python object is finished/destroyed */
+    aws_mqtt_client_connection_set_connection_closed_handler(py_connection->native, NULL, NULL);
+
     aws_mqtt_client_connection_release(py_connection->native);
 
     Py_DECREF(py_connection->self_proxy);
@@ -69,7 +77,10 @@ static void s_mqtt_python_connection_destructor_on_disconnect(
     struct aws_mqtt_client_connection *connection,
     void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
+
     struct mqtt_connection_binding *py_connection = userdata;
 
     PyGILState_STATE state;
@@ -87,6 +98,9 @@ static void s_mqtt_python_connection_destructor(PyObject *connection_capsule) {
         PyCapsule_GetPointer(connection_capsule, s_capsule_name_mqtt_client_connection);
     assert(py_connection);
 
+    /* This is the destructor from Python - so we can ignore the closed callback here */
+    aws_mqtt_client_connection_set_connection_closed_handler(py_connection->native, NULL, NULL);
+
     if (aws_mqtt_client_connection_disconnect(
             py_connection->native, s_mqtt_python_connection_destructor_on_disconnect, py_connection)) {
 
@@ -97,7 +111,9 @@ static void s_mqtt_python_connection_destructor(PyObject *connection_capsule) {
 
 static void s_on_connection_interrupted(struct aws_mqtt_client_connection *connection, int error_code, void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct mqtt_connection_binding *py_connection = userdata;
 
@@ -126,6 +142,10 @@ static void s_on_connection_resumed(
     bool session_present,
     void *userdata) {
 
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
+
     (void)connection;
 
     struct mqtt_connection_binding *py_connection = userdata;
@@ -147,6 +167,54 @@ static void s_on_connection_resumed(
         }
     }
 
+    /* call _on_connection_success */
+    PyObject *success_result =
+        PyObject_CallMethod(self, "_on_connection_success", "(iN)", return_code, PyBool_FromLong(session_present));
+    if (success_result) {
+        Py_DECREF(success_result);
+    } else {
+        PyErr_WriteUnraisable(PyErr_Occurred());
+    }
+
+    PyGILState_Release(state);
+}
+
+static void s_on_connection_closed(
+    struct aws_mqtt_client_connection *connection,
+    struct on_connection_closed_data *data,
+    void *userdata) {
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
+    (void)data; // Not used for anything currently, but in the future it could be.
+
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
+    struct mqtt_connection_binding *py_connection = userdata;
+    /* Ensure that python class is still alive */
+    PyObject *self = PyWeakref_GetObject(py_connection->self_proxy); /* borrowed reference */
+    if (self != Py_None) {
+        PyObject *result = PyObject_CallMethod(self, "_on_connection_closed", "()");
+        if (result) {
+            Py_DECREF(result);
+        } else {
+            PyErr_WriteUnraisable(PyErr_Occurred());
+        }
+    }
+    Py_DECREF(py_connection->self_proxy);
+
+    /** Allow the PyCapsule to be freed like normal again.
+     * If this is the last reference (I.E customer code called disconnect and threw the Python object away)
+     * Then this will allow the MQTT311 class to be fully cleaned.
+     * If it is not the last reference (customer still has reference) then when the customer is done
+     * it will be freed like normal.
+     **/
+    Py_DECREF(py_connection->self_capsule);
+
     PyGILState_Release(state);
 }
 
@@ -193,6 +261,12 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
         goto set_interruption_failed;
     }
 
+    if (aws_mqtt_client_connection_set_connection_closed_handler(
+            py_connection->native, s_on_connection_closed, py_connection)) {
+        PyErr_SetAwsLastError();
+        goto set_interruption_failed;
+    }
+
     if (PyObject_IsTrue(use_websocket_py)) {
         if (aws_mqtt_client_connection_use_websockets(
                 py_connection->native,
@@ -219,6 +293,7 @@ PyObject *aws_py_mqtt_client_connection_new(PyObject *self, PyObject *args) {
 
     /* From hereon, nothing will fail */
 
+    py_connection->self_capsule = capsule;
     py_connection->self_proxy = self_proxy;
 
     py_connection->client = client_py;
@@ -253,15 +328,18 @@ static void s_on_connect(
     bool session_present,
     void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct mqtt_connection_binding *py_connection = user_data;
 
+    PyGILState_STATE state;
+    if (aws_py_gilstate_ensure(&state)) {
+        return; /* Python has shut down. Nothing matters anymore, but don't crash */
+    }
+
     if (py_connection->on_connect) {
-        PyGILState_STATE state;
-        if (aws_py_gilstate_ensure(&state)) {
-            return; /* Python has shut down. Nothing matters anymore, but don't crash */
-        }
 
         PyObject *callback = py_connection->on_connect;
         py_connection->on_connect = NULL;
@@ -275,15 +353,42 @@ static void s_on_connect(
         }
 
         Py_XDECREF(callback);
-
-        PyGILState_Release(state);
     }
+
+    /* Call on_connection_success or failure based on the result */
+    PyObject *self = PyWeakref_GetObject(py_connection->self_proxy); /* borrowed reference */
+    if (self != Py_None) {
+        /* Successful connection - call _on_connection_success */
+        if (error_code == AWS_ERROR_SUCCESS) {
+            PyObject *success_result = PyObject_CallMethod(
+                self, "_on_connection_success", "(iN)", return_code, PyBool_FromLong(session_present));
+            if (success_result) {
+                Py_DECREF(success_result);
+            } else {
+                PyErr_WriteUnraisable(PyErr_Occurred());
+            }
+            /* Unsuccessful connection - call _on_connection_failure */
+        } else {
+            PyObject *success_result = PyObject_CallMethod(self, "_on_connection_failure", "(i)", error_code);
+            if (success_result) {
+                Py_DECREF(success_result);
+            } else {
+                PyErr_WriteUnraisable(PyErr_Occurred());
+            }
+        }
+    }
+
+    PyGILState_Release(state);
 }
 
 /* If unsuccessful, false is returned and a Python error has been set */
 bool s_set_will(struct aws_mqtt_client_connection *connection, PyObject *will) {
     assert(will && (will != Py_None));
 
+    if (connection == NULL) {
+        return false; // The connection is dead - skip!
+    }
+
     bool success = false;
 
     /* These references all need to be cleaned up before function returns */
@@ -697,7 +802,10 @@ static void s_publish_complete(
     uint16_t packet_id,
     int error_code,
     void *userdata) {
-    (void)connection;
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     struct publish_complete_userdata *metadata = userdata;
     assert(metadata);
@@ -805,7 +913,9 @@ static void s_subscribe_callback(
     bool retain,
     void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = user_data;
     if (callback == Py_None) {
@@ -858,7 +968,9 @@ static void s_suback_callback(
     int error_code,
     void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = userdata;
     AWS_FATAL_ASSERT(callback && callback != Py_None);
@@ -967,7 +1079,10 @@ static void s_unsuback_callback(
     uint16_t packet_id,
     int error_code,
     void *userdata) {
-    (void)connection;
+
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *callback = userdata;
 
@@ -1029,7 +1144,9 @@ static void s_suback_multi_callback(
     int error_code,
     void *userdata) {
 
-    (void)connection;
+    if (connection == NULL || userdata == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     /* These must be DECREF'd when function ends */
     PyObject *callback = userdata;
@@ -1063,7 +1180,7 @@ static void s_suback_multi_callback(
             goto done_prepping_args;
         }
 
-        PyList_SET_ITEM(topic_qos_list, i, tuple); /* Steals reference to tuple */
+        PyList_SetItem(topic_qos_list, i, tuple); /* Steals reference to tuple */
     }
 
 done_prepping_args:;
@@ -1124,7 +1241,9 @@ PyObject *aws_py_mqtt_client_connection_resubscribe_existing_topics(PyObject *se
 
 static void s_on_disconnect(struct aws_mqtt_client_connection *connection, void *user_data) {
 
-    (void)connection;
+    if (connection == NULL || user_data == NULL) {
+        return; // The connection is dead - skip!
+    }
 
     PyObject *on_disconnect = user_data;
 
@@ -1163,24 +1282,31 @@ PyObject *aws_py_mqtt_client_connection_disconnect(PyObject *self, PyObject *arg
     }
 
     Py_INCREF(on_disconnect);
+    Py_INCREF(connection->self_proxy);   /* We need to keep self_proxy alive for on_closed, which will dec-ref this */
+    Py_INCREF(connection->self_capsule); /* Do not allow the PyCapsule to be freed, we need it alive for on_closed */
 
     int err = aws_mqtt_client_connection_disconnect(connection->native, s_on_disconnect, on_disconnect);
     if (err) {
         Py_DECREF(on_disconnect);
+        Py_DECREF(connection->self_proxy);
+        Py_DECREF(connection->self_capsule);
         return PyErr_AwsLastError();
     }
 
     Py_RETURN_NONE;
 }
 
+/*******************************************************************************
+ * Client Statistics
+ ******************************************************************************/
+
 PyObject *aws_py_mqtt_client_connection_get_stats(PyObject *self, PyObject *args) {
     (void)self;
     bool success = false;
 
     PyObject *impl_capsule;
-    PyObject *get_stats_callback_fn_py;
 
-    if (!PyArg_ParseTuple(args, "OO", &impl_capsule, &get_stats_callback_fn_py)) {
+    if (!PyArg_ParseTuple(args, "O", &impl_capsule)) {
         return NULL;
     }
 
@@ -1190,7 +1316,7 @@ PyObject *aws_py_mqtt_client_connection_get_stats(PyObject *self, PyObject *args
         return NULL;
     }
 
-    /* These must be DECREF'd when function ends */
+    /* These must be DECREF'd when function ends on error */
     PyObject *result = NULL;
 
     struct aws_mqtt_connection_operation_statistics stats;
@@ -1198,26 +1324,49 @@ PyObject *aws_py_mqtt_client_connection_get_stats(PyObject *self, PyObject *args
 
     aws_mqtt_client_connection_get_stats(connection->native, &stats);
 
-    result = PyObject_CallFunction(
-        get_stats_callback_fn_py,
-        "(KKKK)",
-        /* K */ (unsigned long long)stats.incomplete_operation_count,
-        /* K */ (unsigned long long)stats.incomplete_operation_size,
-        /* K */ (unsigned long long)stats.unacked_operation_count,
-        /* K */ (unsigned long long)stats.unacked_operation_size);
+    result = PyTuple_New(4);
     if (!result) {
-        PyErr_WriteUnraisable(PyErr_Occurred());
+        goto done;
+    }
+
+    PyTuple_SetItem(
+        result,
+        0,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_count)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 0) == NULL) {                                               /* Borrowed reference */
+        goto done;
+    }
+
+    PyTuple_SetItem(
+        result,
+        1,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.incomplete_operation_size)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 1) == NULL) {                                              /* Borrowed reference */
+        goto done;
+    }
+
+    PyTuple_SetItem(
+        result,
+        2,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_count)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 2) == NULL) {                                            /* Borrowed reference */
+        goto done;
+    }
+
+    PyTuple_SetItem(
+        result,
+        3,
+        PyLong_FromUnsignedLongLong((unsigned long long)stats.unacked_operation_size)); /* Steals a reference */
+    if (PyTuple_GetItem(result, 3) == NULL) {                                           /* Borrowed reference */
         goto done;
     }
 
     success = true;
 
 done:
-
-    Py_XDECREF(result);
-
     if (success) {
-        Py_RETURN_NONE;
+        return result;
     }
+    Py_XDECREF(result);
     return NULL;
 }
